"""
Convert Coordenadas

"""


def dms2dd_daee(coord):
    """
    Convert geographic coordinates in
    format degrees, minutes and seconds (22° 55' 09'')
    in decimal degrees (e.g. -22.91916666666667)

    Estilo DAEE Hidrologia

    :param coord: string 23° 06' 09''
    :return: float -22.91916666666667
    """
    # Splitar coordenada
    coord = coord.replace("''", '')
    coord = coord.replace("'", '-')
    coord = coord.replace("°", '-')
    coord = coord.replace(' ', '')
    coord = coord.split('-')
    graus = float(coord[0])
    minutos = float(coord[1])
    segundos = float(coord[2])

    # Calcular
    coord_dm = graus + (minutos / 60) + (segundos / 3600)

    # Converter parâmetro textual
    return coord_dm * -1


def dms2dd(coord):
    """
    Convert geographic coordinates in
    format degrees, minutes and seconds (23°06’12,48”S)
    in decimal degrees (e.g. -23.10346666666667)

    Estilo Informações Técnicas CETESB

    :param coord: string 23°06’12,48”S
    :return: float -23.10346666666667
    """
    # Splitar coordenada
    graus = float(coord.split('°')[0])
    minutos = float((coord.split('°')[1]).split('’')[0])
    segundos = float(
        (((coord.split('°')[1]).split('’')[1]).split('”')[0]).replace(',', '.'))
    direction = (((coord.split('°')[1]).split('’')[1]).split('”')[1])

    # Calcular
    coord_dm = graus + (minutos / 60) + (segundos / 3600)

    # Converter parâmetro textual
    if direction in ('S', 's', 'O', 'o'):
        return coord_dm * -1
    else:
        return coord_dm


def dms2dd_infoaguas(coord_dms):
    """
    Para usar em uma coluna
    df.loc[:, 'latitude_dd'] = df['latitude_dms'].astype(str).apply(lambda x: dms2dd(x))

    :param coord_dms:
    :return:
    """
    coord_dms = coord_dms.strip()
    try:
        coord_deg = float(coord_dms.split(' ', maxsplit=2)[0])
        coord_min = float(coord_dms.split(' ', maxsplit=2)[1])
        coord_sec = float(coord_dms.split(' ', maxsplit=2)[2])
        coord_dd = coord_deg + (coord_min / 60) + (coord_sec / 3600)
        coord_dd = coord_dd * -1
    except:
        coord_dd = 0
    return coord_dd


if __name__ == '__main__':
    print(dms2dd('23°06’12,48”S'))
    print(dms2dd_infoaguas('22 13 52'))
    print(dms2dd_daee("22° 55' 09''"))



"""
Provides open_geodata version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update open_geodata` to change this file.

from incremental import Version

__version__ = Version("open_geodata", 22, 6, 60)
__all__ = ["__version__"]



"""
sssss
"""

import importlib.resources
import pprint
import tempfile
from pathlib import Path

import geopandas as gpd
import pandas as pd
import py7zr
from more_itertools import one, only


def _ajust_list_files(list_files):
    """

    :param list_files:
    :return:
    """
    # Convert to text
    list_files = [str(x.as_posix()) for x in list_files]

    # Splita no . e pega primeira parte
    list_files = [x.split('.', maxsplit=1)[0] for x in list_files]

    # Convert Paths Linux
    list_files = [x.replace('/', '.') for x in list_files]
    list_files = list(set(list_files))
    list_files.sort()
    return list_files


def _read_7z_file(file_path_7z):
    if file_path_7z.is_file():
        with py7zr.SevenZipFile(file_path_7z, 'r') as archive:
            allfiles = archive.getnames()

            # Quero apenas um arquivo por gpkg
            if len(allfiles) == 1:
                with tempfile.TemporaryDirectory() as temp_dir:
                    temp_dir = Path(temp_dir)
                    archive.extract(path=temp_dir, targets=allfiles)
                    return gpd.read_file(list(temp_dir.glob('*'))[0])

            else:
                raise RuntimeError('.zip tem mais de um gpkg')


def get_dataset_names():
    """
    fdfddfd

    """
    package_path = Path(__file__).absolute().parent
    data_path = package_path / 'data'
    list_files = data_path.rglob('*.*')
    list_files = [x for x in list_files]
    list_files = [x.relative_to(data_path) for x in list_files]
    return _ajust_list_files(list_files)


def load_dataset(dataset_name) -> pd.DataFrame | gpd.GeoDataFrame:
    """
    Funções para carregar dados geoespaciais

    :param dataset_name:
    :return:
    """

    filename = dataset_name.replace('.', '/')
    filename = Path(filename)
    package_path = Path(__file__).absolute().parent
    data_path = package_path / 'data'
    file_path = data_path / filename

    # Checa se existe
    list_shp = get_dataset_names()
    if dataset_name not in list_shp:
        raise RuntimeError(f'"{filename}" not exists')

    # Checa se existe mais de um
    if list_shp.count(dataset_name) > 1:
        raise RuntimeError(
            f'Exists "{list_shp.count(dataset_name)}" datasets named "{dataset_name}"'
        )

    # Teste ambos tipos de arquivos
    file_path_7z = file_path.with_suffix('.7z')
    file_path_csv = file_path.with_suffix('.csv')

    # Load por tipo de arquivo
    if file_path_7z.is_file():
        return _read_7z_file(file_path_7z)

    # Se o arquivo é um
    elif file_path_csv.is_file():
        return pd.read_csv(file_path_csv)

    else:
        print('Não encontrado')


def get_dataset_from_package(package_name):
    """
    Pega dados dos pacotes
    Os dados preciso estar disponibilizados em .7z ou .csv

    """
    # sss
    package_path = importlib.resources.files(package_name)
    list_files = [
        x for x in package_path.rglob('*') if x.suffix in ('.7z', '.csv')
    ]
    list_files = [x.relative_to(package_path) for x in list_files]
    list_files = [x.relative_to('data') for x in list_files]
    for path in ['input', 'output']:
        try:
            list_files = [x.relative_to(path) for x in list_files]
        except:
            pass
    return _ajust_list_files(list_files)


def load_dataset_from_package(package_name, dataset_name):
    """ """
    # Importa o Package
    package_path = importlib.resources.files(package_name)
    print(package_path)

    #
    filename = dataset_name.replace('.', '/')
    filename = Path(filename)
    print(f'O nome do arquivo é {filename}')

    # Lista de Arquivo
    list_7zips = list(package_path.rglob(f'{filename}*.7z'))
    list_csv = list(package_path.rglob(f'{filename}*.csv'))

    # Pega Valor Único
    file_path_7z = only(list_7zips)
    file_path_csv = only(list_csv)

    # Prints
    # print(f'A lista de 7zip é: {file_path_7z}')
    # print(f'A lista de csv é: {file_path_csv}')

    if file_path_7z is not None and file_path_7z.is_file():
        return _read_7z_file(file_path_7z)

    elif file_path_csv is not None and file_path_csv.is_file():
        return pd.read_csv(file_path_csv)

    else:
        print('Não encontrado')


if __name__ == '__main__':
    from open_geodata import geo
    from open_geodata.functions import find_neighbors, share_boundary

    # # List Geodata
    list_shp = get_dataset_names()
    pprint.pprint(list_shp)

    # # Read Geaodata
    gdf = load_dataset('geo.sp.sp_250k_wgs84')
    print(gdf.head())

    # # List Geodata
    list_shp = get_dataset_from_package('sp_piracicaba')
    pprint.pprint(list_shp)

    gdf = load_dataset_from_package('sp_piracicaba', 'geo.divisa_municipal')
    print(gdf.head())

    # Teste "find_neighbors" attribute table
    # gdf = find_neighbors(gdf, 'municipio_nome')

    # Teste "find_neighbors" spatial
    # gdf_interest = gdf.loc[gdf['id_municipio'] == 3548906]
    # gdf = share_boundary(gdf, gdf_interest)

    # Results
    # print(gdf.head())

    # df = load_dataset('tab_municipio_ugrhi')
    # print(df.head())

    # create_colors(
    #     os.path.join('outorgas.gpkg'),
    #     col_categories = 'uso'
    # )
    # for i in list_shp:
    #     #print(i.parents[2])
    #     a = i.relative_to(i.parents[2])
    #     print(a)



""" """

import py7zr
from pathlib import Path


import geopandas as gpd
from shapely.geometry import Point


def df2geojson(
    df, lat='latitude', long='longitude', remove_coords_properties=True
):
    """
    Convert um dataframe, com colunas de latitude e longitude, em um objeto geojson de pontos
    https://notebook.community/gnestor/jupyter-renderers/notebooks/nteract/pandas-to-geojson

    # Usage
    feature_collection = dataframe2geojson(df, lat='latitude_dd', long='longitude_dd', remove_coords_properties=False)

    with open('file.geojson', 'w', encoding='utf-8') as f:
        json.dump(feature_collection, f, ensure_ascii=False)


    :param df:
    :param lat: Nome da coluna no dataframe que tem os dados de latitude
    :param long: Nome da coluna no dataframe que tem os dados de longitude
    :param remove_coords_properties:
    :return:
    """
    print(
        'Estudar:\nhttps://geopandas.org/en/stable/docs/reference/api/geopandas.points_from_xy.html'
    )
    # Create a new python dict to contain our geojson data, using geojson format
    geojson = {'type': 'FeatureCollection', 'features': []}

    # Loop through each row in the dataframe and convert each row to geojson format
    for _, row in df.iterrows():
        # Create a feature template to fill in
        feature = {
            'type': 'Feature',
            'properties': {},
            'geometry': {
                'type': 'Point',
                'coordinates': [],
            },
        }

        # Fill in the coordinates
        feature['geometry']['coordinates'] = [row[long], row[lat]]

        # for each column, get the value and add it as a new feature property
        properties = list(df.columns)
        if remove_coords_properties:
            properties.remove(lat)
            properties.remove(long)

        for prop in properties:
            feature['properties'][prop] = row[prop]

        # Add this feature (aka, converted dataframe row) to the list of features inside our dict
        geojson['features'].append(feature)

    return geojson


def df2gdf(df, lat='latitude', long='longitude', epsg=4326):
    """

    :param epsg:
    :param df:
    :param lat:
    :param long:
    :param remove_coords_properties:
    :return:
    """
    # Create Geometry
    geometry = [Point(xy) for xy in zip(df[long], df[lat])]

    # Create Geodataframe
    gdf = gpd.GeoDataFrame(df, crs=f'EPSG:{epsg}', geometry=geometry)
    gdf.drop([lat, long], axis=1, inplace=True, errors='ignore')
    return gdf


def convert_to_7zip(input_path, output_path, extension='gpkg'):
    """

    :return:
    """
    list_files = list(input_path.rglob(f'*.{extension}'))
    if len(list_files) == 0:
        raise RuntimeError(
            f'Não existem arquivos "{extension}" na pasta {input_path}'
        )

    for file in list_files:
        # Paths
        zip7_filepath = output_path / f'{file.stem}.7z'

        # Write 7zip
        with py7zr.SevenZipFile(zip7_filepath, 'w') as archive:
            archive.write(file, file.name)

        # Print
        print(f'Arquivo "{file.name}" convertido para "{file.stem}.7z"')

    print(f'Foram convertidos {len(list_files)} arquivos com sucesso!')
    return 0


if __name__ == '__main__':

    # Convert
    data_path = Path(__file__).parents[1].joinpath('data')
    ibge_path = data_path / 'geo' / 'br_ibge'
    convert_to_7zip(ibge_path, ibge_path)

    pass



"""


"""




import folium


def google_hybrid(min_zoom, max_zoom):
    row = {
        'link': 'https://mt1.google.com/vt/lyrs=y&x={x}&y={y}&z={z}',
        'name': 'Google Hybrid',
        'attribution': 'https://www.google.com/maps',
    }
    return folium.TileLayer(
        tiles=row['link'],
        attr=('<a href="{}" target="blank">{}</a>'.format(row['attribution'], row['name'])),
        name=row['name'],
        min_zoom=min_zoom,
        max_zoom=max_zoom,
        subdomains=['mt0', 'mt1', 'mt2', 'mt3'],
        overlay=False,
        control=True,
        show=True,
    )


def google_satellite(min_zoom, max_zoom):
    row = {
        'link': 'https://mt1.google.com/vt/lyrs=s&x={x}&y={y}&z={z}',
        'name': 'Google Satelite',
        'attribution': 'https://www.google.com/maps',
    }
    return folium.TileLayer(
        tiles=row['link'],
        attr=('<a href="{}" target="blank">{}</a>'.format(row['attribution'], row['name'])),
        name=row['name'],
        min_zoom=min_zoom,
        max_zoom=max_zoom,
        subdomains=['mt0', 'mt1', 'mt2', 'mt3'],
        overlay=False,
        control=True,
        show=False,
    )


def google_terrain(min_zoom, max_zoom):
    row = {
        'link': 'https://mt1.google.com/vt/lyrs=p&x={x}&y={y}&z={z}',
        'name': 'Google Terrain',
        'attribution': 'https://www.google.com/maps',
    }
    return folium.TileLayer(
        tiles=row['link'],
        attr=('<a href="{}" target="blank">{}</a>'.format(row['attribution'], row['name'])),
        name=row['name'],
        min_zoom=min_zoom,
        max_zoom=max_zoom,
        subdomains=['mt0', 'mt1', 'mt2', 'mt3'],
        overlay=False,
        control=True,
        show=False,
    )


def google_streets(min_zoom, max_zoom):
    row = {
        'link': 'https://mt1.google.com/vt/lyrs=m&x={x}&y={y}&z={z}',
        'name': 'Google Streets',
        'attribution': 'https://www.google.com/maps',
    }
    return folium.TileLayer(
        tiles=row['link'],
        attr=('<a href="{}" target="blank">{}</a>'.format(row['attribution'], row['name'])),
        name=row['name'],
        min_zoom=min_zoom,
        max_zoom=max_zoom,
        subdomains=['mt0', 'mt1', 'mt2', 'mt3'],
        overlay=False,
        control=True,
        show=False,
    )


def cartodb_positron(min_zoom, max_zoom):
    return folium.TileLayer(
        tiles='cartodbpositron',
        attr='Carto',
        name='CartoDB Positron',
        min_zoom=min_zoom,
        max_zoom=max_zoom,
        overlay=False,
        control=True,
        show=False,
    )


if __name__ == '__main__':
    pass
